/****************************************************************************
** Meta object code from reading C++ file 'cattributeitemwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../frame/AttributesWidgets/private/cattributeitemwidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cattributeitemwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CAttributeWgt::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CAttributeWgt",
        "attriChanged",
        "",
        "QVariant",
        "var",
        "phase"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'attriChanged'
        QtMocHelpers::SignalData<void(const QVariant, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'attriChanged'
        QtMocHelpers::SignalData<void(const QVariant)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CAttributeWgt, qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CAttributeWgt::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CAttributeWgt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CAttributeWgt *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->attriChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->attriChanged((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CAttributeWgt::*)(const QVariant , int )>(_a, &CAttributeWgt::attriChanged, 0))
            return;
    }
}

const QMetaObject *DrawAttribution::CAttributeWgt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CAttributeWgt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution13CAttributeWgtE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DrawAttribution::CAttributeWgt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DrawAttribution::CAttributeWgt::attriChanged(const QVariant _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CExpButton::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CExpButton",
        "clicked",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CExpButton, qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CExpButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CExpButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CExpButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CExpButton::*)()>(_a, &CExpButton::clicked, 0))
            return;
    }
}

const QMetaObject *DrawAttribution::CExpButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CExpButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution10CExpButtonE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DrawAttribution::CExpButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DrawAttribution::CExpButton::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution7CSplineE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CSpline::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution7CSplineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CSpline"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CSpline, qt_meta_tag_ZN15DrawAttribution7CSplineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CSpline::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution7CSplineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution7CSplineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution7CSplineE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CSpline::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CSpline *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DrawAttribution::CSpline::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CSpline::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution7CSplineE_t>.strings))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int DrawAttribution::CSpline::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CExpWgt::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CExpWgt"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CExpWgt, qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CExpWgt::staticMetaObject = { {
    QMetaObject::SuperData::link<DBlurEffectWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CExpWgt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CExpWgt *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DrawAttribution::CExpWgt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CExpWgt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution7CExpWgtE_t>.strings))
        return static_cast<void*>(this);
    return DBlurEffectWidget::qt_metacast(_clname);
}

int DrawAttribution::CExpWgt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DBlurEffectWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CAttriBaseOverallWgt::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CAttriBaseOverallWgt"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CAttriBaseOverallWgt, qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CAttriBaseOverallWgt::staticMetaObject = { {
    QMetaObject::SuperData::link<CAttributeWgt::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CAttriBaseOverallWgt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CAttriBaseOverallWgt *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DrawAttribution::CAttriBaseOverallWgt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CAttriBaseOverallWgt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution20CAttriBaseOverallWgtE_t>.strings))
        return static_cast<void*>(this);
    return CAttributeWgt::qt_metacast(_clname);
}

int DrawAttribution::CAttriBaseOverallWgt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CAttributeWgt::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CColorSettingButton::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CColorSettingButton",
        "colorChanged",
        "",
        "QColor",
        "color",
        "phase",
        "setColor",
        "setDefaultButtonSize",
        "QSize",
        "sz",
        "setColorFill",
        "EColorFill",
        "fillPos"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(const QColor &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Slot 'setColor'
        QtMocHelpers::SlotData<void(const QColor &)>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setDefaultButtonSize'
        QtMocHelpers::SlotData<void(const QSize &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'setColorFill'
        QtMocHelpers::SlotData<void(enum EColorFill)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CColorSettingButton, qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CColorSettingButton::staticMetaObject = { {
    QMetaObject::SuperData::link<CAttributeWgt::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CColorSettingButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CColorSettingButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 2: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->setColor((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 4: _t->setDefaultButtonSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 5: _t->setColorFill((*reinterpret_cast<std::add_pointer_t<enum EColorFill>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CColorSettingButton::*)(const QColor & , int )>(_a, &CColorSettingButton::colorChanged, 0))
            return;
    }
}

const QMetaObject *DrawAttribution::CColorSettingButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CColorSettingButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution19CColorSettingButtonE_t>.strings))
        return static_cast<void*>(this);
    return CAttributeWgt::qt_metacast(_clname);
}

int DrawAttribution::CColorSettingButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CAttributeWgt::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DrawAttribution::CColorSettingButton::colorChanged(const QColor & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CComBoxSettingWgt::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CComBoxSettingWgt"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CComBoxSettingWgt, qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CComBoxSettingWgt::staticMetaObject = { {
    QMetaObject::SuperData::link<CAttributeWgt::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CComBoxSettingWgt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CComBoxSettingWgt *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *DrawAttribution::CComBoxSettingWgt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CComBoxSettingWgt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution17CComBoxSettingWgtE_t>.strings))
        return static_cast<void*>(this);
    return CAttributeWgt::qt_metacast(_clname);
}

int DrawAttribution::CComBoxSettingWgt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CAttributeWgt::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CSpinBoxSettingWgt::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CSpinBoxSettingWgt",
        "widthChanged",
        "",
        "width"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CSpinBoxSettingWgt, qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CSpinBoxSettingWgt::staticMetaObject = { {
    QMetaObject::SuperData::link<CAttributeWgt::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CSpinBoxSettingWgt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CSpinBoxSettingWgt *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->widthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CSpinBoxSettingWgt::*)(int )>(_a, &CSpinBoxSettingWgt::widthChanged, 0))
            return;
    }
}

const QMetaObject *DrawAttribution::CSpinBoxSettingWgt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CSpinBoxSettingWgt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution18CSpinBoxSettingWgtE_t>.strings))
        return static_cast<void*>(this);
    return CAttributeWgt::qt_metacast(_clname);
}

int DrawAttribution::CSpinBoxSettingWgt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CAttributeWgt::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DrawAttribution::CSpinBoxSettingWgt::widthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawAttribution::CGroupButtonWgt::qt_create_metaobjectdata<qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawAttribution::CGroupButtonWgt",
        "buttonClicked",
        "",
        "doGroup",
        "doUngroup"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'buttonClicked'
        QtMocHelpers::SignalData<void(bool, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::Bool, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CGroupButtonWgt, qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawAttribution::CGroupButtonWgt::staticMetaObject = { {
    QMetaObject::SuperData::link<CAttributeWgt::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t>.metaTypes,
    nullptr
} };

void DrawAttribution::CGroupButtonWgt::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CGroupButtonWgt *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->buttonClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CGroupButtonWgt::*)(bool , bool )>(_a, &CGroupButtonWgt::buttonClicked, 0))
            return;
    }
}

const QMetaObject *DrawAttribution::CGroupButtonWgt::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawAttribution::CGroupButtonWgt::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15DrawAttribution15CGroupButtonWgtE_t>.strings))
        return static_cast<void*>(this);
    return CAttributeWgt::qt_metacast(_clname);
}

int DrawAttribution::CGroupButtonWgt::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CAttributeWgt::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DrawAttribution::CGroupButtonWgt::buttonClicked(bool _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
