/****************************************************************************
** Meta object code from reading C++ file 'ccentralwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../frame/ccentralwidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ccentralwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4PageE_t {};
} // unnamed namespace

template <> constexpr inline auto Page::qt_create_metaobjectdata<qt_meta_tag_ZN4PageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Page"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Page, qt_meta_tag_ZN4PageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Page::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4PageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4PageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4PageE_t>.metaTypes,
    nullptr
} };

void Page::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Page *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Page::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Page::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4PageE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Page::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9DrawBoardE_t {};
} // unnamed namespace

template <> constexpr inline auto DrawBoard::qt_create_metaobjectdata<qt_meta_tag_ZN9DrawBoardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DrawBoard",
        "pageAdded",
        "",
        "Page*",
        "page",
        "pageRemoved",
        "currentPageChanged",
        "pageKey",
        "currentTitleNameChange",
        "modified",
        "zoomValueChanged",
        "totalScaled",
        "toClose",
        "addPage",
        "name",
        "QImage",
        "img",
        "PageContext*",
        "pageCxt",
        "closePage",
        "key",
        "setCurrentPage",
        "const Page*",
        "setDrawAttribution",
        "attris",
        "QVariant",
        "var",
        "phase",
        "autoCmdStack",
        "onFileContextChanged",
        "path",
        "tp"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pageAdded'
        QtMocHelpers::SignalData<void(Page *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'pageRemoved'
        QtMocHelpers::SignalData<void(Page *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'currentPageChanged'
        QtMocHelpers::SignalData<void(Page *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'currentPageChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'currentTitleNameChange'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modified'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'zoomValueChanged'
        QtMocHelpers::SignalData<void(qreal)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 },
        }}),
        // Signal 'toClose'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addPage'
        QtMocHelpers::SlotData<Page *(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'addPage'
        QtMocHelpers::SlotData<Page *()>(13, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 3),
        // Slot 'addPage'
        QtMocHelpers::SlotData<Page *(const QImage &)>(13, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'addPage'
        QtMocHelpers::SlotData<Page *(PageContext *)>(13, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'addPage'
        QtMocHelpers::SlotData<void(Page *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'closePage'
        QtMocHelpers::SlotData<bool(const QString &)>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'closePage'
        QtMocHelpers::SlotData<bool(Page *)>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'setCurrentPage'
        QtMocHelpers::SlotData<void(const Page *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 4 },
        }}),
        // Slot 'setCurrentPage'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'setDrawAttribution'
        QtMocHelpers::SlotData<void(int, const QVariant &, int, bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { 0x80000000 | 25, 26 }, { QMetaType::Int, 27 }, { QMetaType::Bool, 28 },
        }}),
        // Slot 'setDrawAttribution'
        QtMocHelpers::SlotData<void(int, const QVariant &, int)>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { 0x80000000 | 25, 26 }, { QMetaType::Int, 27 },
        }}),
        // Slot 'setDrawAttribution'
        QtMocHelpers::SlotData<void(int, const QVariant &)>(23, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 24 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'onFileContextChanged'
        QtMocHelpers::SlotData<void(const QString &, int)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::Int, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DrawBoard, qt_meta_tag_ZN9DrawBoardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DrawBoard::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DrawBoardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DrawBoardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9DrawBoardE_t>.metaTypes,
    nullptr
} };

void DrawBoard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DrawBoard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pageAdded((*reinterpret_cast<std::add_pointer_t<Page*>>(_a[1]))); break;
        case 1: _t->pageRemoved((*reinterpret_cast<std::add_pointer_t<Page*>>(_a[1]))); break;
        case 2: _t->currentPageChanged((*reinterpret_cast<std::add_pointer_t<Page*>>(_a[1]))); break;
        case 3: _t->currentPageChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->currentTitleNameChange(); break;
        case 5: _t->modified((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->zoomValueChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->toClose(); break;
        case 8: { Page* _r = _t->addPage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Page**>(_a[0]) = std::move(_r); }  break;
        case 9: { Page* _r = _t->addPage();
            if (_a[0]) *reinterpret_cast<Page**>(_a[0]) = std::move(_r); }  break;
        case 10: { Page* _r = _t->addPage((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Page**>(_a[0]) = std::move(_r); }  break;
        case 11: { Page* _r = _t->addPage((*reinterpret_cast<std::add_pointer_t<PageContext*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Page**>(_a[0]) = std::move(_r); }  break;
        case 12: _t->addPage((*reinterpret_cast<std::add_pointer_t<Page*>>(_a[1]))); break;
        case 13: { bool _r = _t->closePage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->closePage((*reinterpret_cast<std::add_pointer_t<Page*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setCurrentPage((*reinterpret_cast<std::add_pointer_t<const Page*>>(_a[1]))); break;
        case 16: _t->setCurrentPage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->setDrawAttribution((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 18: _t->setDrawAttribution((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 19: _t->setDrawAttribution((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 20: _t->onFileContextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Page* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Page* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Page* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PageContext* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Page* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Page* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)(Page * )>(_a, &DrawBoard::pageAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)(Page * )>(_a, &DrawBoard::pageRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)(Page * )>(_a, &DrawBoard::currentPageChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)(const QString & )>(_a, &DrawBoard::currentPageChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)()>(_a, &DrawBoard::currentTitleNameChange, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)(bool )>(_a, &DrawBoard::modified, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)(qreal )>(_a, &DrawBoard::zoomValueChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DrawBoard::*)()>(_a, &DrawBoard::toClose, 7))
            return;
    }
}

const QMetaObject *DrawBoard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DrawBoard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9DrawBoardE_t>.strings))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int DrawBoard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void DrawBoard::pageAdded(Page * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DrawBoard::pageRemoved(Page * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DrawBoard::currentPageChanged(Page * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DrawBoard::currentPageChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DrawBoard::currentTitleNameChange()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DrawBoard::modified(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DrawBoard::zoomValueChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DrawBoard::toClose()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
