/****************************************************************************
** Meta object code from reading C++ file 'cgraphicsview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../frame/cgraphicsview.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cgraphicsview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8PageViewE_t {};
} // unnamed namespace

template <> constexpr inline auto PageView::qt_create_metaobjectdata<qt_meta_tag_ZN8PageViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PageView",
        "signalSetScale",
        "",
        "scale",
        "signalTransmitContinueDoOtherThing",
        "singalTransmitEndLoadDDF",
        "updateSelectedItemsAlignment",
        "Qt::AlignmentFlag",
        "align",
        "alignmentMovPos",
        "QPointF",
        "QRectF",
        "fatherRect",
        "childRect",
        "slotOnCut",
        "slotOnCopy",
        "slotOnPaste",
        "textItemInCenter",
        "slotOnSelectAll",
        "slotOnDelete",
        "slotOneLayerUp",
        "slotOneLayerDown",
        "slotBringToFront",
        "slotSendTobackAct",
        "slotOnTextCut",
        "slotOnTextCopy",
        "slotOnTextPaste",
        "slotOnTextSelectAll",
        "slotSetTextAlignment",
        "Qt::Alignment",
        "slotOnTextUndo",
        "slotOnTextRedo",
        "slotOnTextDelete",
        "slotViewZoomIn",
        "slotViewZoomOut",
        "slotViewOrignal"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalSetScale'
        QtMocHelpers::SignalData<void(const qreal)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 3 },
        }}),
        // Signal 'signalTransmitContinueDoOtherThing'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'singalTransmitEndLoadDDF'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSelectedItemsAlignment'
        QtMocHelpers::SlotData<void(Qt::AlignmentFlag)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'alignmentMovPos'
        QtMocHelpers::SlotData<QPointF(const QRectF &, const QRectF &, Qt::AlignmentFlag)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { 0x80000000 | 11, 12 }, { 0x80000000 | 11, 13 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'slotOnCut'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnCopy'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnPaste'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'slotOnPaste'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotOnSelectAll'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnDelete'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOneLayerUp'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOneLayerDown'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotBringToFront'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSendTobackAct'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnTextCut'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnTextCopy'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnTextPaste'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnTextSelectAll'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetTextAlignment'
        QtMocHelpers::SlotData<void(const Qt::Alignment &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 8 },
        }}),
        // Slot 'slotOnTextUndo'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnTextRedo'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotOnTextDelete'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotViewZoomIn'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotViewZoomOut'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotViewOrignal'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PageView, qt_meta_tag_ZN8PageViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PageView::staticMetaObject = { {
    QMetaObject::SuperData::link<DGraphicsView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8PageViewE_t>.metaTypes,
    nullptr
} };

void PageView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PageView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalSetScale((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 1: _t->signalTransmitContinueDoOtherThing(); break;
        case 2: _t->singalTransmitEndLoadDDF(); break;
        case 3: _t->updateSelectedItemsAlignment((*reinterpret_cast<std::add_pointer_t<Qt::AlignmentFlag>>(_a[1]))); break;
        case 4: { QPointF _r = _t->alignmentMovPos((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::AlignmentFlag>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->slotOnCut(); break;
        case 6: _t->slotOnCopy(); break;
        case 7: _t->slotOnPaste((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->slotOnPaste(); break;
        case 9: _t->slotOnSelectAll(); break;
        case 10: _t->slotOnDelete(); break;
        case 11: _t->slotOneLayerUp(); break;
        case 12: _t->slotOneLayerDown(); break;
        case 13: _t->slotBringToFront(); break;
        case 14: _t->slotSendTobackAct(); break;
        case 15: _t->slotOnTextCut(); break;
        case 16: _t->slotOnTextCopy(); break;
        case 17: _t->slotOnTextPaste(); break;
        case 18: _t->slotOnTextSelectAll(); break;
        case 19: _t->slotSetTextAlignment((*reinterpret_cast<std::add_pointer_t<Qt::Alignment>>(_a[1]))); break;
        case 20: _t->slotOnTextUndo(); break;
        case 21: _t->slotOnTextRedo(); break;
        case 22: _t->slotOnTextDelete(); break;
        case 23: _t->slotViewZoomIn(); break;
        case 24: _t->slotViewZoomOut(); break;
        case 25: _t->slotViewOrignal(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PageView::*)(const qreal )>(_a, &PageView::signalSetScale, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::signalTransmitContinueDoOtherThing, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::singalTransmitEndLoadDDF, 2))
            return;
    }
}

const QMetaObject *PageView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageViewE_t>.strings))
        return static_cast<void*>(this);
    return DGraphicsView::qt_metacast(_clname);
}

int PageView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void PageView::signalSetScale(const qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PageView::signalTransmitContinueDoOtherThing()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PageView::singalTransmitEndLoadDDF()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
